////////////////////////////////////////////////////////////////////////////////
//
// Copyright 2018 The University of North Carolina at Chapel Hill.
// All Rights Reserved.
//
// Permission to use, copy, modify and distribute this software and its
// documentation for educational, research and non-profit purposes, without
// fee, and without a written agreement is hereby granted, provided that the
// above copyright notice and the following three paragraphs appear in all
// copies.
//
// IN NO EVENT SHALL THE UNIVERSITY OF NORTH CAROLINA AT CHAPEL HILL BE
// LIABLE TO ANY PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR
// CONSEQUENTIAL DAMAGES, INCLUDING LOST PROFITS, ARISING OUT OF THE
// USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF THE UNIVERSITY
// OF NORTH CAROLINA HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH
// DAMAGES.
//
// THE UNIVERSITY OF NORTH CAROLINA SPECIFICALLY DISCLAIM ANY
// WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
// PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
// NORTH CAROLINA HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT,
// UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
//
// The authors may be contacted via:
//
// US Mail: Real-Time Systems Group at UNC
// Department of Computer Science
// Sitterson Hall
// University of N. Carolina
// Chapel Hill, NC 27599-3175
//
// EMail: nemitz@cs.unc.edu; tamert@cs.unc.edu; anderson@cs.unc.edu
//
////////////////////////////////////////////////////////////////////////////////

#ifndef CRNLP_UNIFORM_H 
#define CRNLP_UNIFORM_H

#include <stdbool.h>
#include <stdint.h>
#include "nesting.h"
#include "cycles.h"
#include "queue.h"
#include "setup.h"

typedef struct {
    resource_mask_t requested;
    uint32_t        length;
    uint_fast8_t    next;
} crnlp_uniform_node_t;

typedef struct
{
    __attribute__((aligned(64))) volatile uint_fast8_t enabled[SCHEDSIZE];

    crnlp_uniform_node_t         requests[QUEUE_LENGTH];

    resource_mask_t              schedule[SCHEDSIZE];

    uint_fast8_t                 blocked[SCHEDSIZE];
    uint_fast8_t                 waiting[SCHEDSIZE];

    uint_fast8_t                 pending_requests;

    uint_fast8_t                 head;
    uint_fast8_t                 head_time;

    mcs_lock_t                   sublock;
} crnlp_uniform_lock_t;

void crnlp_uniform_init(crnlp_uniform_lock_t* lock);

uint64_t *crnlp_uniform_lock(crnlp_uniform_lock_t* lock, uint8_t ind);
uint64_t *crnlp_uniform_unlock(crnlp_uniform_lock_t* lock, uint8_t ind);

#endif // CRNLP_UNIFORM_H
