import sys

from numpy import percentile
from numpy import loadtxt
import glob
import os
import csv
import re

# TODO: Set processor speed in MHz
procSpeed = 2100.000

def main():
    if len(sys.argv) != 3:
        print >> sys.stderr, "Usage: to_csv.py directory out.csv"
        sys.exit(1)

    regexp = re.compile('.*/(.*)-(.*)-(.*)-(.*)-(.*).csv') # numcores-numresources-numrequested-cslength-ALG
    
    with open(sys.argv[2], 'wb') as outfile:
        fieldnames = ['numcores', 'numresources', 'numrequested',
                      'lock', 'unlock', 'blocking',
                      'cslength']

        writer = csv.DictWriter(outfile, fieldnames = fieldnames)
        writer.writeheader()

        for file in glob.glob(sys.argv[1]+"/*.csv"):
            #print file
            row = {}
            m = regexp.match(file)
            row['numcores']     = m.group(1)
            row['numresources'] = m.group(2)
            row['numrequested'] = m.group(3)
            row['cslength']     = m.group(4)

            try:
                data = loadtxt(file, skiprows=1, delimiter=",")
            except:
                print "Failed to load file:", file

            row['lock'] = percentile(data[:,4]/procSpeed, 99)
            row['unlock'] = percentile(data[:,5]/procSpeed, 99)
            row['blocking'] = percentile(data[:,6]/procSpeed, 99)

            writer.writerow(row)

if __name__ == '__main__':
    main()
