#include "sharedres.h"
#include "blocking.h"
#include "rw-blocking.h"
#include <iostream>

#include "stl-helper.h"

bool share_resources(const std::vector<RequestBound> r_list1, const std::vector<RequestBound> r_list2)
{
    unsigned int i, j;
    for(i = 0; i < r_list1.size(); i++)
    {
        for(j = 0; j < r_list2.size(); j++)
        {
            if(r_list1[i].get_resource_id() == r_list2[j].get_resource_id())
            {
                return true;
            }
        }
    }
    return false;
}

// Check if r_list1 is a subset of r_list2
bool is_subset(const std::vector<RequestBound> r_list1, const std::vector<RequestBound> r_list2)
{
    unsigned int i, j;
    for(i = 0; i < r_list1.size(); i++)
    {
        bool found = false;
        for(j = 0; j < r_list2.size(); j++)
        {
            if (r_list1[i].get_resource_id() == r_list2[j].get_resource_id())
            {
                found = true;
                break;
            }
        }

        if (!found)
        {
            return false;
        }
    }
    return true;
}

unsigned int get_contention(const TaskInfo &tsk,
                            const std::vector<TaskInfo> &all_tasks)
{
    std::vector<const RequestBound*> conflicting_reqs;
    unsigned int i;
    unsigned int c = 0;
    for(i = 0; i < all_tasks.size(); i++)
    {
        if(tsk.get_id() != all_tasks[i].get_id())
        {
            if(share_resources(tsk.get_requests(), all_tasks[i].get_requests()))
            {
                c++;
            }
        }
    }
    return c;
}

BlockingBounds* ucrnlp_bounds(const ResourceSharingInfo& info,
				 unsigned int procs_per_cluster,
				 int dedicated_irq)
{
    // split everything by partition
    Clusters clusters;
    split_by_cluster(info, clusters);

    // split each partition by resource
    ClusterResources resources;

    split_by_resource(clusters, resources);

    // sort each contention set by request length
    sort_by_request_length(resources);

    std::vector<const RequestBound*> all_reqs;
    unsigned int k = 0;
    for(k = 0; k < resources[0].size(); k++)
    {
        foreach(resources[0][k], it)
        {
            const RequestBound* req = *it;
            unsigned int m = 0;
            bool req_already_recorded = false;
            for(m=0; m<all_reqs.size(); m++)
            {
                if(all_reqs[m]->get_task() == req->get_task())
                {
                    req_already_recorded = true;
                    break;
                }
            }
            if (req_already_recorded == false)
            {
                all_reqs.push_back(req);
            }
        }
    }
    sort_by_request_length(all_reqs);


    // We need for each task the maximum request span.  We also need the
    // maximum direct blocking from remote partitions for each request. We
    // can determine both in one pass.

    unsigned int i;

    // direct blocking results
    BlockingBounds* _results = new BlockingBounds(info);
    BlockingBounds& results = *_results;

    for (i = 0; i < info.get_tasks().size(); i++)
    {
        const TaskInfo& tsk  = info.get_tasks()[i];
        Interference bterm;

        if (tsk.get_requests().size() > 0)
        {
            const RequestBound* this_req = &tsk.get_requests()[0];

            Interference wblocking, wblocking_w1;

            int idx;
            unsigned int parallelism = procs_per_cluster;
	    enumerate(resources, ct, idx)
            {
                parallelism = procs_per_cluster;

                if (idx == dedicated_irq)
                {
                    parallelism--;
                }

                if (parallelism && (int) tsk.get_cluster() == idx)
                {
                    parallelism--;
                }
            }

            // Calculate blocking
            if (parallelism == 0)
            {
                wblocking = 0;
            }
            else
            {
                unsigned int contention = get_contention(tsk,info.get_tasks());
                unsigned int count = 0;
                unsigned int total_length = 0;
                // Goal (Ci + 1)(Lmax)
                unsigned int num_remaining = std::min(contention + 1, parallelism);

                // Find num_remaining jobs that block me (all_reqs is ordered
                // by decreasing critical-section length)
                foreach(all_reqs, it)
                {
                    const RequestBound* req = *it;
                    if(num_remaining == 0)
                    {
                        break;
                    }
                    if(req->get_task() != &tsk)
                    {
                        unsigned int num;

                        // If my request is a subset of this other one, then a FIFO
                        // constraint applies (num=1); otherwise, it's a period-based constraint
                        if (is_subset(tsk.get_requests(), req->get_task()->get_requests()))
                        {
                            num = 1;
                        }
                        else
                        {
                            num = std::min(req->get_max_num_requests(tsk.get_response()),
                                           num_remaining);
                        }

                        total_length += num*req->get_request_length();
                        count += num;
                        num_remaining -= num;
                    }
                }
                wblocking.total_length = total_length;
                wblocking.count = count;
            }

            wblocking_w1 = wblocking;
            wblocking_w1.total_length += this_req->get_request_length();
            wblocking_w1.count += 1;

            results.raise_request_span(i, wblocking_w1);
            bterm += wblocking;
        }
        results[i] = bterm;
    }

    charge_arrival_blocking(info, results);

    return _results;
}
